﻿using System;

namespace UMK.WzorceStrukturalne
{
    //component
    interface INapój
    {
        decimal Cena();
    }

    //concrete components
    class Herbata : INapój
    {
        public decimal Cena()
        {
            return 6.70M;
        }

        public override string ToString()
        {
            return "Herbata";
        }
    }

    class Kawa : INapój
    {
        public decimal Cena()
        {
            return 7.90M;
        }

        public override string ToString()
        {
            return "Kawa";
        }
    }

    //decorator
    abstract class NapójZDodatkiem : INapój //TODO: czy to może być interfejsem
    {
        protected INapój napój;

        public abstract decimal Cena();

        public NapójZDodatkiem(INapój napój)
        {
            this.napój = napój;
        }
    }

    //concrete decorator
    class NapójZeSzczyptąCynamonu : NapójZDodatkiem
    {
        public NapójZeSzczyptąCynamonu(INapój napój)
            : base(napój)
        {
        }

        public override decimal Cena()
        {
            return napój.Cena() + 0.5M;
        }

        public override string ToString()
        {
            return napój.ToString() + " ze szczyptą cynamonu";
        }
    }

    class NapójZCukrem : NapójZDodatkiem
    {
        private int liczbaŁyżeczek;

        public NapójZCukrem(INapój napój, int liczbaŁyżeczek = 1)
            : base(napój)
        {
            this.liczbaŁyżeczek = liczbaŁyżeczek;
        }

        public override decimal Cena()
        {
            return napój.Cena() + liczbaŁyżeczek * 0.2M;
        }

        public override string ToString()
        {
            return napój.ToString() + $" z {liczbaŁyżeczek} łyżeczkami cukru";
        }
    }

    class NapójZPlastremCytryny : NapójZDodatkiem
    {
        public NapójZPlastremCytryny(INapój napój)
            : base(napój) { }

        public override decimal Cena()
        {
            return napój.Cena() + 900000M;
        }

        public override string ToString()
        {
            return napój.ToString() + $" z plastrem cytryny";
        }
    }
}